# Generated by roxygen2: do not edit by hand

export(PEAXAI_counterfactuals)
export(PEAXAI_fitting)
export(PEAXAI_global_importance)
export(PEAXAI_local_importance)
export(PEAXAI_peer)
export(PEAXAI_predict)
export(PEAXAI_ranking)
export(reffcy)
importFrom(Benchmarking,dea.add)
importFrom(PRROC,pr.curve)
importFrom(caret,confusionMatrix)
importFrom(caret,createDataPartition)
importFrom(caret,createFolds)
importFrom(caret,train)
importFrom(caret,trainControl)
importFrom(deaR,make_deadata)
importFrom(deaR,maximal_friends)
importFrom(deaR,model_additive)
importFrom(dplyr,"%>%")
importFrom(dplyr,across)
importFrom(dplyr,all_of)
importFrom(dplyr,anti_join)
importFrom(dplyr,arrange)
importFrom(dplyr,bind_cols)
importFrom(dplyr,bind_rows)
importFrom(dplyr,filter)
importFrom(dplyr,first)
importFrom(dplyr,group_by)
importFrom(dplyr,select)
importFrom(dplyr,semi_join)
importFrom(dplyr,summarise)
importFrom(iml,FeatureImp)
importFrom(iml,Predictor)
importFrom(kernelshap,kernelshap)
importFrom(lime,lime)
importFrom(np,npudens)
importFrom(np,npudensbw)
importFrom(pROC,auc)
importFrom(pROC,roc)
importFrom(peakRAM,peakRAM)
importFrom(rminer,Importance)
importFrom(rms,val.prob)
importFrom(stats,approxfun)
importFrom(stats,binomial)
importFrom(stats,family)
importFrom(stats,glm)
importFrom(stats,median)
importFrom(stats,na.omit)
importFrom(stats,predict)
importFrom(stats,quantile)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,setNames)
importFrom(utils,combn)
importFrom(utils,head)
importFrom(utils,tail)
