% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convex_facets.R
\name{convex_facets}
\alias{convex_facets}
\title{Identify Maximal Facets of the Convex Frontier}
\usage{
convex_facets(data, x, y, RTS = "vrs")
}
\arguments{
\item{data}{A \code{data.frame} or \code{matrix} containing the variables used in the model.}

\item{x}{Integer vector indicating the column indexes of the input variables in \code{data}.}

\item{y}{Integer vector indicating the column indexes of the output variables in \code{data}.}

\item{RTS}{Text string or number defining the underlying DEA technology /
returns-to-scale assumption (default: \code{"vrs"}). Accepted values:
\describe{
\item{\code{0} / \code{"fdh"}}{Free disposability hull, no convexity assumption.}
\item{\code{1} / \code{"vrs"}}{Variable returns to scale, convexity and free disposability.}
\item{\code{2} / \code{"drs"}}{Decreasing returns to scale, convexity, down-scaling and free disposability.}
\item{\code{3} / \code{"crs"}}{Constant returns to scale, convexity and free disposability.}
\item{\code{4} / \code{"irs"}}{Increasing returns to scale (up-scaling, not down-scaling), convexity and free disposability.}
\item{\code{5} / \code{"add"}}{Additivity (scaling up and down, but only with integers), and free disposability.}
}}
}
\value{
A \code{matrix} where each row represents a maximal facet, and the values are the row indices of the efficient DMUs in \code{data} that form that facet. Returns an empty \code{matrix} if there are no efficient units.
}
\description{
Identifies the maximal subsets of efficient Decision-Making Units (DMUs)
that lie on the same face (facet) of the efficient frontier. It relies on
the concept of "maximal friends" to determine which efficient units can
be grouped together to form convex combinations under the chosen returns-to-scale assumption.
}
